@@ -45,6 +45,9 @@ from .complete import main_bashcomplete, complete_path, CompleteUnhandled
 # Some globals resolved for default arguments in the cli
 build_stream_version = pkg_resources.require("buildstream")[0].version
 
+# Intendation for all logging
+INDENT = 4
+
 
 ##################################################################
 #            Override of click's main entry point                #
@@ -215,15 +218,14 @@ def build(app, elements, all, track, track_save, track_all, track_except):
     if track_all:
         track = elements
 
-    app.initialize(elements, except_=track_except, rewritable=track_save)
-    app.pipeline.initialize(use_remote_cache=True, inconsistent=track)
+    app.initialize(elements, except_=track_except, rewritable=track_save,
+                   use_remote_cache=True, inconsistent=track)
     app.print_heading()
     try:
         app.pipeline.build(app.scheduler, all, track, track_save)
-        click.echo("", err=True)
         app.print_summary()
-    except PipelineError:
-        click.echo("", err=True)
+    except PipelineError as e:
+        app.print_error(e)
         app.print_summary()
         sys.exit(-1)
 
@@ -259,16 +261,15 @@ def fetch(app, elements, deps, track, except_):
         all:   All dependencies
     """
 
-    app.initialize(elements, except_=except_, rewritable=track)
-    app.pipeline.initialize(inconsistent=elements if track else None)
+    app.initialize(elements, except_=except_, rewritable=track,
+                   inconsistent=elements if track else None)
     try:
         dependencies = app.pipeline.deps_elements(deps)
         app.print_heading(deps=dependencies)
         app.pipeline.fetch(app.scheduler, dependencies, track)
-        click.echo("", err=True)
         app.print_summary()
     except PipelineError as e:
-        click.echo("{}".format(e), err=True)
+        app.print_error(e)
         app.print_summary()
         sys.exit(-1)
 
@@ -299,16 +300,14 @@ def track(app, elements, deps, except_):
         none:  No dependencies, just the element itself
         all:   All dependencies
     """
-    app.initialize(elements, except_=except_, rewritable=True)
-    app.pipeline.initialize(inconsistent=elements)
+    app.initialize(elements, except_=except_, rewritable=True, inconsistent=elements)
     try:
         dependencies = app.pipeline.deps_elements(deps)
         app.print_heading(deps=dependencies)
         app.pipeline.track(app.scheduler, dependencies)
-        click.echo("", err=True)
         app.print_summary()
     except PipelineError as e:
-        click.echo("{}".format(e), err=True)
+        app.print_error(e)
         app.print_summary()
         sys.exit(-1)
 
@@ -332,15 +331,14 @@ def pull(app, elements, deps):
         none:  No dependencies, just the element itself
         all:   All dependencies
     """
-    app.initialize(elements)
-    app.pipeline.initialize(use_remote_cache=True)
+    app.initialize(elements, use_remote_cache=True)
     try:
         to_pull = app.pipeline.deps_elements(deps)
         app.pipeline.pull(app.scheduler, to_pull)
-        click.echo("", err=True)
+        app.print_summary()
     except BstError as e:
-        click.echo("", err=True)
-        click.echo("ERROR: {}".format(e), err=True)
+        app.print_error(e)
+        app.print_summary()
         sys.exit(-1)
 
 
@@ -363,15 +361,14 @@ def push(app, elements, deps):
         none:  No dependencies, just the element itself
         all:   All dependencies
     """
-    app.initialize(elements)
-    app.pipeline.initialize(use_remote_cache=True)
+    app.initialize(elements, use_remote_cache=True)
     try:
         to_push = app.pipeline.deps_elements(deps)
         app.pipeline.push(app.scheduler, to_push)
-        click.echo("", err=True)
+        app.print_summary()
     except BstError as e:
-        click.echo("", err=True)
-        click.echo("ERROR: {}".format(e), err=True)
+        app.print_error(e)
+        app.print_summary()
         sys.exit(-1)
 
 
@@ -443,8 +440,7 @@ def show(app, elements, deps, except_, order, format, downloadable):
         bst show target.bst --format \\
             $'---------- %{name} ----------\\n%{vars}'
     """
-    app.initialize(elements, except_=except_)
-    app.pipeline.initialize(use_remote_cache=downloadable)
+    app.initialize(elements, except_=except_, use_remote_cache=downloadable)
     try:
         dependencies = app.pipeline.deps_elements(deps)
     except PipelineError as e:
@@ -497,7 +493,6 @@ def shell(app, element, sysroot, build, command):
         scope = Scope.RUN
 
     app.initialize((element,))
-    app.pipeline.initialize()
 
     # Assert we have everything we need built.
     missing_deps = []
@@ -542,13 +537,11 @@ def checkout(app, element, directory, force, integrate, hardlinks):
     """Checkout a built artifact to the specified directory
     """
     app.initialize((element,))
-    app.pipeline.initialize()
     try:
         app.pipeline.checkout(directory, force, integrate, hardlinks)
         click.echo("", err=True)
     except BstError as e:
-        click.echo("")
-        click.echo("ERROR: {}".format(e), err=True)
+        app.print_error(e)
         sys.exit(-1)
 
 
@@ -574,8 +567,7 @@ def checkout(app, element, directory, force, integrate, hardlinks):
 def source_bundle(app, target, force, directory,
                   track, compression, except_):
     """Produce a source bundle to be manually executed"""
-    app.initialize((target,), rewritable=track)
-    app.pipeline.initialize(inconsistent=[target])
+    app.initialize((target,), rewritable=track, inconsistent=[target])
     try:
         dependencies = app.pipeline.deps_elements('all')
         app.print_heading(dependencies)
@@ -616,8 +608,7 @@ def workspace():
 def workspace_open(app, no_checkout, force, source, track, element, directory):
     """Open a workspace for manual source modification"""
 
-    app.initialize((element,), rewritable=track)
-    app.pipeline.initialize(inconsistent=[element])
+    app.initialize((element,), rewritable=track, inconsistent=[element])
     try:
         app.pipeline.open_workspace(app.scheduler, directory, source, no_checkout, track, force)
         click.echo("", err=True)
@@ -642,7 +633,6 @@ def workspace_close(app, source, remove_dir, element):
     """Close a workspace"""
 
     app.initialize((element,))
-    app.pipeline.initialize()
     if app.interactive and remove_dir:
         if not click.confirm('This will remove all your changes, are you sure?'):
             click.echo('Aborting', err=True)
@@ -673,7 +663,6 @@ def workspace_close(app, source, remove_dir, element):
 def workspace_reset(app, source, track, no_checkout, element):
     """Reset a workspace to its original state"""
     app.initialize((element,))
-    app.pipeline.initialize()
     if app.interactive:
         if not click.confirm('This will remove all your changes, are you sure?'):
             click.echo('Aborting', err=True)
@@ -781,7 +770,7 @@ class App():
     # Initialize the main pipeline
     #
     def initialize(self, elements, except_=tuple(), rewritable=False,
-                   inconsistent=False, use_remote_cache=False):
+                   inconsistent=None, use_remote_cache=False):
 
         profile_start(Topics.LOAD_PIPELINE, "_".join(t.replace(os.sep, '-') for t in elements))
 
@@ -836,7 +825,7 @@ class App():
             self.error_profile,
             self.detail_profile,
             # Indentation for detailed messages
-            indent=4,
+            indent=INDENT,
             # Number of last lines in an element's log to print (when encountering errors)
             log_lines=self.context.log_error_lines,
             # Maximum number of lines to print in a detailed message
@@ -866,6 +855,13 @@ class App():
                              self.pipeline, self.scheduler,
                              colors=self.colors)
 
+        # Initialize pipeline
+        try:
+            self.pipeline.initialize(use_remote_cache=use_remote_cache, inconsistent=inconsistent)
+        except BstError as e:
+            click.echo("Error initializing pipeline: {}".format(e), err=True)
+            sys.exit(-1)
+
         # Pipeline is loaded, lets start displaying pipeline messages from tasks
         self.logger.size_request(self.pipeline)
 
@@ -1039,10 +1035,22 @@ class App():
     # Print a summary of the queues
     #
     def print_summary(self):
+        click.echo("", err=True)
         self.logger.print_summary(self.pipeline, self.scheduler,
                                   self.main_options['log_file'],
                                   styling=self.colors)
 
+    #
+    # Print an error
+    #
+    def print_error(self, error):
+        click.echo("", err=True)
+        click.echo("{}".format(error), err=True)
+        if error.detail:
+            indent = " " * INDENT
+            detail = '\n' + indent + indent.join(error.detail.splitlines(True))
+            click.echo("{}".format(detail), err=True)
+
     #
     # Handle messages from the pipeline
     #
