@@ -80,11 +80,14 @@ class Scope(Enum):
 
 
 class ElementError(BstError):
-    """Raised by Element implementations.
+    """This exception should be raised by :class:`.Element` implementations
+    to report errors to the user.
 
-    This exception is raised when an :class:`.Element` encounters an error.
+    Args:
+       message (str): The error message to report to the user
+       reason (str): An optional machine readable reason string, used for test cases
     """
-    def __init__(self, message, reason=None):
+    def __init__(self, message, *, reason=None):
         super().__init__(message, domain=ErrorDomain.ELEMENT, reason=reason)
 
 
